#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <sys/select.h>
#include <byteswap.h>

#include "fpga.h"
#include "ansi.h"

struct input {
	int base;
	const char *name;
	short prev;
	short period;
} inputs[] = {
	{ 96, "quad0",  0 },
	{ 99, "quad1",  0 },
	{ 102, "quad2", 0 },
	{ 105, "edge0", 0 },
	{ 108, "edge1", 0 },
	{ 111, "edge2", 0 },
	{ 114, "edge3", 0 },
	{ 117, "edge4", 0 },
	{ 120, "edge5", 0 },
};
const int n = sizeof(inputs) / sizeof(struct input);

int main(void)
{
	int c, i, j, ret, k;
	unsigned short s1, s2, v, d;
	fd_set fds;
	struct timeval tv;
	struct termios options, previous_options;
	unsigned int period[6];
	int twifd = fpga_init(NULL, 0);


	if(twifd == -1) {
		perror("Can't open FPGA I2C bus");
		return 1;
	}


	tcgetattr(STDIN_FILENO, &options);
	previous_options = options;
	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	tcsetattr(STDIN_FILENO, TCSANOW, &options);

	memset(period, 0, sizeof(period));
	fpoke8(twifd, 155, 100);
	sleep(1);

	FD_ZERO(&fds);
	FD_SET(STDIN_FILENO,&fds);

	memset(period, 0, sizeof(period));

	ANSI_CLS();

	for(;;) {
		k = 0;
		for(i=0; i < n; i++) {
			s1 = fpeek8(twifd, inputs[i].base);		// read MSB
			v = fpeek8(twifd, inputs[i].base + 1);	// read LSB
			s2 = fpeek8(twifd, inputs[i].base + 2);	// read MSB alias
			if (s1 == s2)
				v |= (s2 << 8);
			else
				v  = (s2 << 8);

			ANSI_GOTO(i+1, 0);

			d = v % 70;

			printf("%s: ", inputs[i].name);

			ANSI_GOTO(i+1, 9);

			for(j=9; j < 80; j++)
				if (j == (d+9))
				putchar('x');
			else
				putchar('-');

			if (inputs[i].prev != v) {
				inputs[i].prev = v;
				k = 1;
				}
		}

		fpeekstream8(twifd, (uint8_t*)period, 131, sizeof(period));

		for(j=0; j < 6; j++) {
			unsigned int x = bswap_32(period[j]);

			ANSI_GOTO(n+j+1, 1);
			ANSI_ERASE_LINE();

			if (x)
				printf("period%d=%u, freq%d=%lld\n", j, x, j, (100LL * 63000000) / x );
			else
				printf("period%d=0\n", j);

		}

		printf("\n\nPress 'A' to reset period-counters\n"
			"Press control-C to quit\n");

		if (k) {
			tv.tv_sec = 0;
			tv.tv_usec = 100000;
		} else {
			tv.tv_sec = 2;   // wait a couple of seconds
			tv.tv_usec = 0;
		}
		FD_ZERO(&fds);
		FD_SET(STDIN_FILENO,&fds);

		ret=select(FD_SETSIZE,&fds,NULL,NULL,&tv);
		if (ret && FD_ISSET(STDIN_FILENO, &fds)) {
			c = getchar();
			if (c == 3) break;

			else if (c == 'a' || c == 'A') {
				fpoke8(twifd, 155, 100);
				sleep(1);
			}

		}
	}


	tcsetattr(STDIN_FILENO, TCSANOW, &previous_options);
	ANSI_GOTO(23, 1);

	return 0;
}